//### shortcuts for cooperation strategy parameters

cool_trustedAgents(Agents) :- 
	cooperationStrategy(trustedAgents(Agents), _, _, _, _, _).

cool_retrievalPolicy(RPolicy) :- 
	cooperationStrategy(_, retrievalPolicy(RPolicy), _, _, _, _).
	
cool_acquisitionPolicy(APolicy) :- 
	cooperationStrategy(_, _, acquisitionPolicy(APolicy), _, _, _).
	
cool_timeout(N) :- 
	cooperationStrategy(_, _, _, _, timeout(N), _).

cool_threshold(T) :- 
	cooperationStrategy(_, _, _, threshold(T), _, _).

cool_alignment_method(M) :- 
	cooperationStrategy(_, _, _, _, _, alignment_method(M)).

/* For profiling purposes */
mydiff(T1, T2, T2-T1).

//### plan without a threshold: anything will work! 
+!askPlan(Trigger, EID) :true <-
	?cool_threshold(T);
	!askPlan(Trigger, EID, T).

+!askPlan(Trigger, EID, T) : true <-
	//custom.stdlib.get_current_ms(MS1);
	//.print("MS1 = ", MS1);
	?cool_trustedAgents(Agents);
	?cool_timeout(N);
	?cool_alignment_method(M);
	// custom.stdlib.generate_hash(Trigger, hash(H));
	//.print("[cool] Asking a plan...");
	//### To each agent in the list of trusted ones
	for ( .member(Ag,Agents)) {
		/* Tell him your confidence threshold */
		.send(Ag, tell, threshold(T));
		/* Tell him the alignment method you want to be used */
		.send(Ag, tell, alignment_method(M));
		/* Send a request for suitable plans */
		.send(Ag, askHow, req(Trigger, EID));
	}
	/* wait for the unfreeze notification from an agent or a given amount of time */
	.concat("+unfreeze(", EID, ")", Ev);
	// .print("[cool] Ev: ", Ev, ElapsedTime);
	.wait(Ev, N, ElapsedTime);
	!replace_plans(Trigger, EID);
	custom.stdlib.unfreeze_event(EID).
	// custom.stdlib.get_current_ms(MS2);
	//.print("MS1 = ", MS2);
	//?mydiff(MS1, MS2, TotalTime);
	//.print("[cool] Total time = ", TotalTime);
	//.print("[cool] end").
	
//### if an ontology has already been registered, do not register it again!
+!registered_ontology(URI, Label) : registered_ontology(URI, Label).

+!replace_plans(Trigger, EID) : cool_acquisitionPolicy(add) <-
	//.print("cool add").
	noop.

+!replace_plans(Trigger, EID) : cool_acquisitionPolicy(replace) <-
	//### Retrieve plans for trigger Trigger
	.relevant_plans(Trigger, ListOfPlans);
	//### If there at least one plan has been sent (identified by the presence of the hash code in its label), remove older ones
	//### (those without that code)
	custom.stdlib.retrieve_deletable_plans(ListOfPlans, EID, D);
	.remove_plan(D).

/**
 * Knowing an ontology means having registered it
 */
known_ont(Ont) :-
	registered_ontology(_, Ont).

//### declarative
+!registered_ontology(URI, Label) : not registered_ontology(URI, Label) <-
	// custom.stdlib.register_ontology_oa(URI, Label, Result);
	register_ontology(URI, Label, Result);
	+registered_ontology(URI, Label).


/**
 * A request for a plan to handle a given event is sent
 */
@kqmlReceivedAskHow_cool
+!kqml_received(Sender, askHow, req(Trigger, EID), MsgId) <-
	// .print("[cool] Received an askHow");
	// .print("[cool] Trigger: ", Trigger);
	// .print("[cool] EID: ", EID);
	/* Extracts the ontology annotation from the trigger */
	custom.stdlib.extract_ontology_label(Trigger, OLabel);
	// .print("[cool] Extracted ontology label ",OLabel);
	!searchAndSendPlans(Sender, Trigger, EID, MsgId, OLabel)[hopeless].

/**
 * The ontology is known: proceeding with standard protocol
 */
@searchAndSendPlans_ontologyKnown
+!searchAndSendPlans(Sender, Trigger, EID, MsgId, OLabel) : known_ont(OLabel) <-
	// .print("known ontology ", OLabel);
	.relevant_plans(Trigger, ListOfPlans);
	//### TODO add the notification including EID!!!
	if (.length(ListOfPlans, L) & L > 0) {
		/* Send the list of adapted plans back to the requesting agent */
		.send(Sender, tellHow, ListOfPlans, MsgId);
		/* Also notify that agent to unfreeze the event */
		.send(Sender, tell, unfreeze(EID));
		//.print("[cool] Plans sent");
	}.

/**
 * The ontology is NOT known: must try an alignment/concept match
 * TODO: search in more than just ONE ontology for a matching concept
 */
@searchAndSendPlans_ontologyNotKnown
+!searchAndSendPlans(Sender, Trigger, EID, MsgId, OLabel) : not known_ont(OLabel) <-
	// .print("unknown ontology...");
	// .print("[cool] OldTrigger: ", Trigger);
	/* Extract the trigger functor */
	custom.stdlib.extract_trigger_functor(Trigger, TFunctor);
	// .print("[cool] Extracted functor ", TFunctor);
	/* Retrieve the alignment method indicated by the agent */
	?alignment_method(M)[source(Sender)];
	/* Retrieve the confidence threshold indicated by the agent */
	?threshold(T)[source(Sender)];
	// ?hash(H)[source(Sender)];
	/* TODO, retrieve ALL known ontologies */
	// ?known_ont(MyOnt);
	for ( known_ont(MyOnt) ) {
		/* Look for a match (an _environment_ action, the match is retrieved with the next command) */
		match_concept(OLabel, TFunctor, MyOnt, M, T);
		?match(OLabel, TFunctor, MyOnt, NewFunctor, Similarity);
		//.print("[cool] Found a matching concept: ", NewFunctor, " with similarity ", Similarity );
		/* TODO check similarity treshold */
		/* Adapt the trigger in order to search for a suitable plan in current agent's plan library */
		custom.stdlib.adapt_trigger(Trigger, NewFunctor, MyOnt, NewTrigger);
		// .print("[cool] NewTrigger: ", NewTrigger);
		/* Actually search for a relevant plan with default internal action */
		.relevant_plans(NewTrigger, ListOfPlans);
		// .print("[cool] Relevant plans: ", ListOfPlans);
		/* Adapt relevant plans so that the asking agent can understand them */
		/* THE MOST COMPLEX ACTION */
		custom.stdlib.adapt_plans(ListOfPlans, TFunctor, AdaptedPlans, OLabel, M, T, EID);
		// .print("[cool] Adapted plans: ", AdaptedPlans);
		if (.length(AdaptedPlans, L) & L > 0) {
			/* Send the list of adapted plans back to the requesting agent */
			.send(Sender, tellHow, AdaptedPlans, MsgId);
			/* Also notify that agent to unfreeze the event */
			.send(Sender, tell, unfreeze(EID));
			// .print("[cool] Plans sent");
		}
	}.

/* OLD STUFF, SHOULD BE DELETED */

//### Not actually needed should be deprecated
+!align_ontologies(O1, O2) <-
	.print("[cool] Aligning ontologies ", O1, " and ", O2);
	custom.stdlib.align_ontologies(O1, O2);
	+aligned(O1, O2);
	.print("[cool] done").
	
+!ctest : true <-
	.print("just a test!").
