package custom.asSemantics;

import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Queue;

import custom.asSyntax.EnhancedPlanLibrary;

import jasdl.JASDLParams;
import jasdl.asSemantics.JASDLAgent;
import jason.JasonException;
import jason.architecture.AgArch;
import jason.asSemantics.Agent;
import jason.asSemantics.Event;
import jason.asSemantics.Intention;
import jason.asSemantics.Option;
import jason.asSemantics.TransitionSystem;
import jason.asSemantics.Unifier;
import jason.asSyntax.Atom;
import jason.asSyntax.Literal;
import jason.asSyntax.Plan;
import jason.asSyntax.Pred;
import jason.asSyntax.StringTerm;
import jason.asSyntax.StringTermImpl;
import jason.asSyntax.Structure;
import jason.asSyntax.Term;
import jason.asSyntax.Trigger;
import jason.asSyntax.Trigger.TEOperator;
import jason.asSyntax.Trigger.TEType;
import jason.asSyntax.VarTerm;
import jason.asSyntax.directives.FunctionRegister;
import jason.bb.BeliefBase;
import jason.runtime.Settings;

public class CustomAgent extends JASDLAgent {
	
	private Map<String, Event> frozen_events;
	
	protected String cool_retrieval_policy;
	
	public String getCool_retrieval_policy() {
		return cool_retrieval_policy;
	}

	public void setCool_retrieval_policy(String cool_retrieval_policy) {
		this.cool_retrieval_policy = cool_retrieval_policy;
	}

	public CustomAgent() throws Exception{
		super();
		//### guess a custom plan library was useless after all...
		// this.setPL(new EnhancedPlanLibrary(this));
		frozen_events = new HashMap<String, Event>();

		
	}
	
	/**
	 * Returns the annotation term used to indicate that a request has already been sent for this plan,
	 * so no further requests must be made
	 * @return
	 */
	public static final Literal getHopelessPlanAnnot() {
		return Literal.parseLiteral("hopeless");
	}
	
	/**
	 * 
	 * @return
	 */
	public static final Literal getOntAnnot() {
		return Literal.parseLiteral("ont(X)");
	}
	
	/**
	 * Freeze an event
	 * @param label
	 * @param e
	 */
	public void freezeEvent(String label, Event e) {
		frozen_events.put(label, e);
	}
	
	/**
	 * Returns a previously frozen event
	 * @param label
	 * @return
	 */
	public Event unfreezeEvent(String label) {
		return frozen_events.remove(label);
	}
	
	private static String escapeQuotes(Trigger t) {
		String s = t.toString();
		// s.replaceAll("/\"/", "/\\\"/");
		s = s.replace("\"", "\\\"");
		return s;
	}

	@Override
	public Event selectEvent(Queue<Event> events) {

		for (Event e : events) {
			// System.out.println("Event: " + e);
		}
		
		Event e = super.selectEvent(events);
		//### Check in advance if relevant plans exist
		Trigger te = e.getTrigger();
		List<Plan> plans = this.getPL().getCandidatePlans(te);
		
		//### loads acquisition policy from config
		//### TODO this is done every time an event is selected, may be improved...
		// String cool_retrieval_policy = this.getTS().getSettings().getUserParameter("cool_retrieval_policy");

		//### A request to trusted agents is sent only in certain cases:
		//### - no suitable plans have been found OR the retrieval policy is set to "always"
		//### - the event is marked with an ontology annotation 
		//### - trigger.operator != del AND trigger.type == achieve (why?)
		if ((plans.size() == 0 || getCool_retrieval_policy().equals("always")) && isSEEvent(te) && !te.getOperator().equals(TEOperator.del) && te.getType().equals(TEType.achieve) && !te.getLiteral().hasAnnot(getHopelessPlanAnnot())) {
			//### At this point we know that the agent can't handle that event without help from someone else
			
			//### No relevant plans found, injecting the request plan
			System.out.println("[cool] No relevant plans for event " + e);
			
			//### cloning the original trigger
			Trigger cl = te.clone();
			// why should I remove annotations???
			// cl.getLiteral().clearAnnots();

			//### Generate a hash to univocally identify the event 
	    	String str = cl.toString() + System.currentTimeMillis();
	
			String eid = "h" + Math.abs(str.hashCode());
			
			String askplan_event_string ="askPlan(\"" + escapeQuotes(cl) + "\", " + eid + ")";
			
			Event ev = new Event(new Trigger(TEOperator.add, TEType.achieve, Literal.parseLiteral(askplan_event_string)), null);
			
			//### Adding a "hopeless" annotation to the original trigger to remind that something has been tried
			te.getLiteral().addAnnot(getHopelessPlanAnnot());
			freezeEvent(eid, e);
			
			return ev;
		}
		
		return e;
	}
	
	/**
	 * Returns true if input trigger has an annotation of the form o(ontlabel)
	 * @param te
	 * @return
	 */
	private boolean isSEEvent(Trigger te) {
		if (te.getLiteral().getAnnots() != null) {
			for (Term t : te.getLiteral().getAnnots().getAsList()) {
				if (t.isStructure() && ((Structure)t).getFunctor().equals("o")) {
					return true;
				}
			}
		}
		return false;
	}

	@Override
	/**
	 * Also injects plans to register ontologies
	 */
	public TransitionSystem initAg(AgArch arch, BeliefBase bb, String src, Settings stts) throws JasonException {
		TransitionSystem ts = super.initAg(arch, bb, src, stts);
		
		//### loads and sets retrieval policy
		Literal target = Literal.parseLiteral("cooperationStrategy(_,retrievalPolicy(X),_,_,_,_)");
		
		Unifier un = new Unifier();
		
		Iterator<Literal> a = bb.getCandidateBeliefs(target, un);
		
		//### it must have the cooperation strategy set in its asl file
		Literal l = a.next();

		un.unifies(target, l);
		setCool_retrieval_policy(un.get("X").toString()); 
		
		//### TODO
		//### injects goals for automatically registering ontologies
		String jasdl_ontologies = stts.getUserParameter(JASDLParams.MAS2J_PREFIX + JASDLParams.MAS2J_ONTOLOGIES);
		String[] elems = jasdl_ontologies.split(JASDLParams.DELIM);
		
		//### this does not seem to work
		//this.addInitialGoal(Literal.parseLiteral("ctest"));
		//this.addInitialBel(Literal.parseLiteral("mytrue(a)"));
		
		return ts;
	}
	
	public List<Option> loadMorePlans() {
		return null;
	}
	
}