package custom.env;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.net.Socket;
import java.net.UnknownHostException;

import jason.asSyntax.Atom;
import jason.asSyntax.Literal;
import jason.asSyntax.LiteralImpl;
import jason.asSyntax.StringTerm;
import jason.asSyntax.Structure;
import jason.environment.Environment;

public class OntologyEnvironment extends Environment {

	@Override
	public boolean executeAction(String ag, Structure action) {
		
		if (action.getFunctor().toString().equals("register_ontology")) {
			return register_ontology(ag, ((StringTerm)action.getTerm(0)).getString(), action.getTerm(1).toString());
		}
		
		if (action.getFunctor().toString().equals("match_concept")) {
			//### DEBUG
			// System.out.println("Threshold chosen: " + action.getTerm(4).toString());
			return match_concept(ag, action.getTerm(0).toString(), action.getTerm(1).toString(), action.getTerm(2).toString(), action.getTerm(3).toString(), Float.parseFloat(action.getTerm(4).toString()));
		}
		
		if (action.getFunctor().toString().equals("test")) {
			this.addPercept(ag, Literal.parseLiteral("completed(success)"));
			return true;
		}
		
		// return false;
		//### Every action is allowed
		return true;
	}

	/**
	 * Registers an ontology in the (remote?) Ontology Server
	 * @param agent
	 * @param URI
	 * @param label
	 * @return
	 */
	private boolean register_ontology(String agent, String URI, String label) {

		String fromserver;
		Socket kkSocket;
		try {
			kkSocket = new Socket("localhost", 4444);
			PrintWriter out = new PrintWriter(kkSocket.getOutputStream(), true);
			BufferedReader in = new BufferedReader(new InputStreamReader(
					kkSocket.getInputStream()));

			String request = "REGISTER_ONTOLOGY " + URI + " " + label;
			out.println(request);
			fromserver = in.readLine();

			System.out.println(fromserver);

			// ### cleanup
			out.close();
			in.close();
			kkSocket.close();
		} catch (UnknownHostException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			return false;
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			return false;
		}
		return true;
	}
	
	private boolean match_concept(String ag, String o1_label, String c1, String o2_label, String alignment_method, float threshold) {

		String fromserver;
		Socket kkSocket;
		try {
			kkSocket = new Socket("localhost", 4444);
			PrintWriter out = new PrintWriter(kkSocket.getOutputStream(), true);
			BufferedReader in = new BufferedReader(new InputStreamReader(
					kkSocket.getInputStream()));

			String request = "MATCH_CONCEPT " + o1_label + " " + c1 + " "
					+ o2_label + " " + alignment_method + " " + threshold;
			out.println(request);
			fromserver = in.readLine();

			// ### cleanup
			out.close();
			in.close();
			kkSocket.close();
			
			// System.out.println("[OE]: response from server = " + fromserver);
			//### If no matching concept has been found, return false
			if (fromserver == null || fromserver.equals("null")) {
				return false;
			}

			// ### The response will be something like "matchingConcept 0.9523"
			//### TODO add the possibility to return a list of matches
			String[] parts = fromserver.split(" ");
			String matching_concept = parts[0];
			float strength = Float.valueOf(parts[1]);
			
			//### Building the result
			// Structure res = new Structure("match");
			
			Literal res = new LiteralImpl("match");
			
			res.addTerm(new Atom(o1_label));
			res.addTerm(new Atom(c1));
			res.addTerm(new Atom(o2_label));
			res.addTerm(new Atom(matching_concept));
			res.addTerm(new jason.asSyntax.NumberTermImpl(strength));
			
			this.addPercept(ag, res);

		} catch (UnknownHostException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			return false;
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			return false;
		}

		return true;
	}
}
