//----------------------------------------------------------------------------
// Copyright (C) 2003  Rafael H. Bordini, Jomi F. Hubner, et al.
// 
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
// 
// To contact the authors:
// http://www.inf.ufrgs.br/~bordini
// http://www.das.ufsc.br/~jomi
//----------------------------------------------------------------------------


package custom.stdlib;

import jason.asSemantics.DefaultInternalAction;
import jason.asSemantics.Event;
import jason.asSemantics.TransitionSystem;
import jason.asSemantics.Unifier;
import jason.asSyntax.Atom;
import jason.asSyntax.ListTerm;
import jason.asSyntax.Literal;
import jason.asSyntax.Pred;
import jason.asSyntax.StringTerm;
import jason.asSyntax.StringTermImpl;
import jason.asSyntax.Structure;
import jason.asSyntax.Term;
import jason.asSyntax.Trigger;
import jason.asSyntax.VarTerm;

/**
  <p>Internal action:
  <b><code>custom.stdlib.extract_ontology_label(<i>Trigger</i>, <i>Label</i>)</code></b>.
  </p>
  
  <p>Description: Extracts the ontology label from a trigger</p>
  <p>Examples:<ul> 

  //### TODO add an example
  <li> <code>custom.stdlib.extract_ontology_label(Trigger, Label)</code>: .</li>

  </ul>
  </p>
  
 */
public class adapt_trigger extends DefaultInternalAction {
    
    @Override public int getMinArgs() { return 4; }
    @Override public int getMaxArgs() { return 4; }

    @Override
    public Object execute(TransitionSystem ts, Unifier un, Term[] args) throws Exception {
        // checkArguments(args);
    	
    	// System.out.println("[cool] Changing functor...");
    	
    	//### I'm pretty sure this is an atom
    	Atom newFunctor = (Atom)args[1];
    	Atom newOntology = (Atom)args[2];
    	
    	//### What kind of class does args[0] belong to?
    	//### Looks like it's a VarTerm...
    	//### which contains a StringTermImpl
    	VarTerm oldTrigger_var = (VarTerm) args[0];
    	StringTerm oldTrigger_string = (StringTerm) oldTrigger_var.getValue();
    	Trigger oldTrigger = Trigger.parseTrigger(oldTrigger_string.getString());
    	Pred oldTrigger_pred = (Pred) oldTrigger.getLiteral();
    	ListTerm annots = oldTrigger.getLiteral().getAnnots();
    	
    	// System.out.println("Oldtrigger: " + oldTrigger);
    	
    	//### change the ontology annotation
    	for (Term ann : annots) {
    		// if (ann.)
    		if (((Literal)ann).getFunctor().equals("o")) {
    			annots.remove(ann);
    			break;
    		}
    	}
    	
    	//### Creating the new "o(newOnt)" annotation
    	Structure newOntAnnot = new Structure("o");
    	newOntAnnot.addTerm(newOntology);

    	annots.add(newOntAnnot);
    	
    	oldTrigger.getLiteral().setAnnots(annots);
    	
    	String oldFunctor_string = oldTrigger_pred.getFunctor();
    	String newTrigger_string = oldTrigger.toString().replaceFirst(oldFunctor_string, newFunctor.toString());
    	
    	// System.out.println("newTrigger_string: " + newTrigger_string);
    	
    	// Trigger newTrigger = Trigger.parseTrigger(newTrigger_string);
    	StringTermImpl newTrigger_stringTerm = new StringTermImpl(newTrigger_string);
    	
    	return un.unifies(args[3], newTrigger_stringTerm);
    	// return true;
    }
    
}
