//----------------------------------------------------------------------------
// Copyright (C) 2003  Rafael H. Bordini, Jomi F. Hubner, et al.
// 
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
// 
// To contact the authors:
// http://www.inf.ufrgs.br/~bordini
// http://www.das.ufsc.br/~jomi
//----------------------------------------------------------------------------


package custom.stdlib;

import jason.asSemantics.DefaultInternalAction;
import jason.asSemantics.Event;
import jason.asSemantics.TransitionSystem;
import jason.asSemantics.Unifier;
import jason.asSyntax.Atom;
import jason.asSyntax.ListTerm;
import jason.asSyntax.Literal;
import jason.asSyntax.StringTerm;
import jason.asSyntax.Structure;
import jason.asSyntax.Term;
import jason.asSyntax.Trigger;
import jason.asSyntax.VarTerm;

/**
  <p>Internal action:
  <b><code>custom.stdlib.extract_ontology_label(<i>Trigger</i>, <i>Label</i>)</code></b>.
  </p>
  
  <p>Description: Extracts the ontology label from a trigger</p>
  <p>Examples:<ul> 

  //### TODO add an example
  <li> <code>custom.stdlib.extract_ontology_label(Trigger, Label)</code>: .</li>

  </ul>
  </p>
  
 */
public class extract_ontology_label extends DefaultInternalAction {
    
    @Override public int getMinArgs() { return 2; }
    @Override public int getMaxArgs() { return 2; }

    @Override
    public Object execute(TransitionSystem ts, Unifier un, Term[] args) throws Exception {
        // checkArguments(args);
    	
    	VarTerm l = (VarTerm)args[0];
    	
    	// System.out.println("Retrieving ontology annotation..." + l);
    	
    	StringTerm v = (StringTerm) l.getValue();
    	
    	//### TODO What if the arg is NOT a plan? Can it happen?
    	Trigger t = Trigger.parseTrigger(v.getString());

    	//### Extract annotations
    	ListTerm annots = t.getLiteral().getAnnots();
    	
    	// System.out.println("annots: " + annots);
    	
    	//### Searches the ontology annotation (the one whose functor is "o")
    	//### TODO What if there's more than just one ontology annotation? Is it legal?
    	for (Term tr : annots) {
    		Literal lit = (Literal)tr;
    		if (lit.getFunctor().toString().equals("o")) {
    			return un.unifies(args[1], lit.getTerm(0));
    		}
    	}
    	
    	return false;
    }
    
}
