//----------------------------------------------------------------------------
// Copyright (C) 2003  Rafael H. Bordini, Jomi F. Hubner, et al.
// 
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
// 
// To contact the authors:
// http://www.inf.ufrgs.br/~bordini
// http://www.das.ufsc.br/~jomi
//----------------------------------------------------------------------------


package custom.stdlib;

import jason.asSemantics.DefaultInternalAction;
import jason.asSemantics.TransitionSystem;
import jason.asSemantics.Unifier;
import jason.asSyntax.Atom;
import jason.asSyntax.ListTerm;
import jason.asSyntax.ListTermImpl;
import jason.asSyntax.Plan;
import jason.asSyntax.Pred;
import jason.asSyntax.Term;
import jason.asSyntax.VarTerm;

/**
  <p>Internal action:
  <b><code>custom.stdlib.adapt_plans(<i>Trigger</i>, <i>Label</i>)</code></b>.
  </p>
  
  <p>Description: adapt plans</p>
  <p>Examples:<ul> 

  //### TODO add an example
  <li> <code>custom.stdlib.adapt_plans(ListOfPlans, TFunctor, AdaptedPlans, OLabel);</code>: .</li>

  </ul>
  </p>
  
 */
public class retrieve_deletable_plans extends DefaultInternalAction {
    
    @Override public int getMinArgs() { return 3; }
    @Override public int getMaxArgs() { return 3; }

    @Override
    public Object execute(TransitionSystem ts, Unifier un, Term[] args) throws Exception {
    	
    	//### The list of relevant plans - all these plans still use their original functor, which is not recognized by
    	//### the agent who requested new plans to handle the event
    	VarTerm plans = (VarTerm) args[0];
    	
    	//### 
    	Atom hash = (Atom)((VarTerm) args[1]).getValue();
    	
    	//### I know the first argument is a list of plans
    	ListTerm plans_list = (ListTerm) plans.getValue();
    	
    	//### Creating a new variable to contain the list of adapted plans
    	ListTerm delenda_labels = new ListTermImpl();

    	//### Look for an updated plan
    	boolean found = false;
    	for (Term plan : plans_list) {
    		Plan p = (Plan) plan;
    		Pred l = p.getLabel();
    		if (l.toString().indexOf(hash.toString()) != -1) {
    			found = true;
    			break;
    		}
    	}
    	
    	//### If at least an updated plan is found, mark the older ones to be deleted
    	if (found) {
    		for (Term plan : plans_list) {
        		Plan p = (Plan) plan;
        		Pred l = p.getLabel();
        		if (l.toString().indexOf(hash.toString()) == -1) {
        			delenda_labels.add(l);
        		}
        	}
    	} else {
    		//### do nothing
    	}
    	
      return un.unifies(args[2], delenda_labels);
    }
    
   
    
    
}
