//----------------------------------------------------------------------------
// Copyright (C) 2003  Rafael H. Bordini, Jomi F. Hubner, et al.
// 
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
// 
// To contact the authors:
// http://www.inf.ufrgs.br/~bordini
// http://www.das.ufsc.br/~jomi
//----------------------------------------------------------------------------


package custom.stdlib;

import jason.JasonException;
import jason.asSemantics.ActionExec;
import jason.asSemantics.Circumstance;
import jason.asSemantics.DefaultInternalAction;
import jason.asSemantics.Event;
import jason.asSemantics.Intention;
import jason.asSemantics.TransitionSystem;
import jason.asSemantics.Unifier;
import jason.asSyntax.Literal;
import jason.asSyntax.StringTermImpl;
import jason.asSyntax.Term;
import jason.asSyntax.Trigger;
import jason.asSyntax.Trigger.TEOperator;
import jason.asSyntax.Trigger.TEType;

import custom.asSemantics.CustomAgent;

/**
  <p>Internal action:
  <b><code>.suspend(<i>G</i>)</code></b>.
  
  <p>Description: suspend goals <i>G</i>, i.e., all intentions trying to achieve G will stop 
  running until the internal action <code>.resume</code> change the state of those intentions.  
  A literal <i>G</i>
  is a goal if there is a triggering event <code>+!G</code> in any plan within
  any intention in I, E, PI, or PA.

  <p>Examples:<ul> 

  <li> <code>.suspend(go(1,3))</code>: suspends intentions to go to the location 1,3.
  <li> <code>.suspend</code>: suspends the current intention.

  </ul>

  @see jason.stdlib.suspended
  @see jason.stdlib.intend
  @see jason.stdlib.desire
  @see jason.stdlib.drop_all_desires
  @see jason.stdlib.drop_all_events
  @see jason.stdlib.drop_all_intentions
  @see jason.stdlib.drop_intention
  @see jason.stdlib.drop_desire
  @see jason.stdlib.succeed_goal
  @see jason.stdlib.fail_goal
  @see jason.stdlib.current_intention
  @see jason.stdlib.resume
  
 */
public class unfreeze_event extends DefaultInternalAction {
    
    @Override public int getMinArgs() { return 1; }
    @Override public int getMaxArgs() { return 1; }

    @Override
    public Object execute(TransitionSystem ts, Unifier un, Term[] args) throws Exception {
        // checkArguments(args);
    	
    	// String EventID = ((StringTermImpl)args[0]).toString();
    	String EventID = args[0].toString();
    	System.out.println("Unfreezing event " + EventID);
    	 
        unfreeze(ts, EventID);
        return true;
    }
    
    public void unfreeze(TransitionSystem ts, String pendingId) throws Exception {
    	CustomAgent ag = (CustomAgent) ts.getAg();
        Circumstance C = ts.getC();
        // C.addEvent(C.removePendingEvent(pendingId.getString()));
        C.addEvent(ag.unfreezeEvent(pendingId));
    }
}
