package server;

import java.net.*;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.io.*;

import server.agents.Agent;
import server.alignment.AromaAligner;

public class OntologyServer {
	
	private static OntologyServerWorker worker;
	private static Result result;
	public static final int STARTING_ID = 42;
	private int last_inserted_id;
	
	private List<Agent> agents;
	private Map<String,URI> registered_ontologies;
	private Map<String,String> alignments;
	
	public OntologyServer() {
		agents = new LinkedList<Agent>();
		registered_ontologies = new HashMap<String, URI>();
		alignments = new HashMap<String, String>();
		last_inserted_id = STARTING_ID;
	}
	
	/**
	 * 
	 * @param o1_label
	 * @param o2_label
	 * @param alignment_file
	 */
	public void addAlignment(String o1_label, String o2_label, String alignment_file) {
		alignments.put(o1_label + "_" + o2_label, alignment_file);
	}
	
	/**
	 * 
	 * @param o1_label
	 * @param o2_label
	 * @return
	 */
	public String getAlignmentFile(String o1_label, String o2_label) {
		return alignments.get(o1_label + "_" + o2_label);
	}
	
	/**
	 * Adds an ontology to the list of the registered ones
	 * @param label
	 * @param uri
	 */
	public void registerOntology(String label, URI uri) {
		registered_ontologies.put(label, uri);
	}
	
	/**
	 * 
	 * @param label
	 * @return
	 */
	public URI getOntologyURI(String label) {
		return registered_ontologies.get(label);
	}
		
	public static void main(String[] args) throws IOException {

		ServerSocket serverSocket = null;
		try {
			serverSocket = new ServerSocket(4444);
		} catch (IOException e) {
			System.err.println("Could not listen on port: 4444.");
			System.exit(1);
		}

		OntologyServer OServer = new OntologyServer();
		PrintWriter out;
		BufferedReader in;
		Socket clientSocket = null;

		System.out.println("Server listening, awaiting clients...");

		while (true) {

			try {
				clientSocket = serverSocket.accept();
				System.out.println("Client connected");
				out = new PrintWriter(clientSocket.getOutputStream(), true);
				in = new BufferedReader(new InputStreamReader(clientSocket
						.getInputStream()));
				String request;

				//### Reads the request string
				request = in.readLine();
				
				//### Instantiates the worker object
				worker = new OntologyServerWorker();
				
				//### Handles request and reads the answer from the worker
				result = worker.handleRequest(request, OServer);
				
				//### Sends back the answer
				out.println(result.getAnswer());

				//### cleanup
				out.close();
				in.close();
				clientSocket.close();
				
				//### Checks if the Server must quit
				if (result.mustQuit()) {
					break;
				}
			} catch (IOException e) {
				System.err.println("Accept failed.");
				System.exit(1);
			} catch (URISyntaxException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}

			
		}
		
		//### Closes the socket
		serverSocket.close();
	}
}
