package server.alignment;

import java.io.IOException;
import java.net.URI;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.InputStream;
import java.util.Properties;

public class AromaAligner extends Aligner {

	//### Path to the folder where AROMA is stored (file aroma.jar must be there)
	protected String AROMA_PATH;
	
	public AromaAligner() {
		
		//### Load and read configuration file, which includes the path to the installed AROMA library
		Properties prop = new Properties();
	    String fileName = "oa.config";
	    InputStream is;
		
	    try {
			is = new FileInputStream(fileName);
		    prop.load(is);

			AROMA_PATH = prop.getProperty("oa.aroma_path");
			System.out.println("Ontology Artifact configuration file read, AROMA path = " + AROMA_PATH);

		} catch (FileNotFoundException e) {
			System.out.println("Ontology Artifact configuration file was not found");
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			System.out.println("Ontology Artifact configuration file could not be read properly");
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

	}
	
	@Override
	/**
	 * returns the alignment filename
	 */
	public String align(String o1_label, String o2_label, URI o1_uri, URI o2_uri, String alignment_folder_path) {

		System.out.println("OServer: aligning ontologies " + o1_uri + " ("
				+ o1_label + ") and " + o2_uri + " (" + o2_label + ")");
		
		//### TODO retrieve the path (what if it is an external resource?)
		String o1_path = o1_uri.getPath();
		String o2_path = o2_uri.getPath();
		
		System.out.println("o1_path " + o1_path);
		
		try {
			//### TODO check folder separator for cross platform compatibility
			String filename = alignment_folder_path + '/' + o1_label + "_" + o2_label
					+ "_aroma_alignment.rdf";
			
			String command = "java -jar " + AROMA_PATH + "/aroma.jar " + o1_path + " " + o2_path + " " + filename;
			
			System.out.println("Executing command " + command);
			
			//### Execute the command
			Runtime rt = Runtime.getRuntime();
			Process pr = rt.exec(command);
			//### Since the alignment file is created by an external process we have to wait for it to finish before going on
			pr.waitFor();
			
			return filename;
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (InterruptedException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		// TODO Auto-generated method stub
		return null;
	}

}
